

class buffered_property(object):
    '''Buffer the result of a method on the class instance'''
    def __init__(self, getter, name=None):
        self.name = name or getter.__name__
        self.getter = getter

    def __get__(self, instance, owner):
        if instance is None:  # pragma: no cover
            return self
        instance.__dict__[self.name] = value = self.getter(instance)
        return value





'''
Classes to parse RFC 2388 mime/multipart content
'''
from collections import defaultdict

class MultipartParser(object):
    def __init__(self, content, boundary):
        self.boundary = ('--' + boundary).encode('ascii')
        self.terminator = ('--' + boundary + '--').encode('ascii')
        self.lines = iter(content.splitlines())

    def parse(self):
        data = defaultdict(list)
        for part in self.parse_part():
            if part.disposition != b'form-data':
                raise ValueError('Invalid content type: %r' % part.content_type)
            data[part.opts['name']] = part.content
        return data

    def parse_part(self):
        line = self.lines.next()
        # Scan to next boundary
        while line != self.boundary:
            line = self.lines.next()

        p = Part()
        # Parse headers
        while line:
            name, _, data = line.partition(b':')
            name = name.lower()
            if name == b'content-disposition':
                bits = name.split(b';')
                p.content_type = bits.pop(0)
                for param in bits:
                    k, _, v = param.strip().partition(b'=')
                    p.opts[k] = v
                p.name = opts[b'name']
            elif name == b'content-type':
                p.content_type = data.strip()

        data = []
        line = line.next()
        p.data = b''.join(
            line for line in self.lines

class Part(object):
    '''
    Encapsulates a MIME part
    '''
    def __init__(self):
        self.content_type = None
        self.disposition = None
        self.name = None
        self.opts = {}
        self.content = None
        self.headers = {}




import cgi
from http.cookies import SimpleCookie
from urllib.parse import parse_qs

from .utils.functional import buffered_property

import logging
log = logging.getLogger(__name__)


class Request(object):
    def __init__(self, app, environ):
        self.app = app
        self.environ = environ
        self.path = self.environ.get('PATH_INFO', b'/')
        self.method = self.environ.get('REQUEST_METHOD')

        self.content_type, self.content_params = cgi.parse_header(environ.get('CONTENT_TYPE', ''))

    @buffered_property
    def raw_cookies(self):
        '''Raw access to cookies'''
        cookie_data = self.environ.get('HTTP_COOKIE', '')
        cookies = SimpleCookie()
        if not cookie_data:
            return cookies
        cookies.load(cookie_data)
        return cookies

    @buffered_property
    def cookies(self):
        '''Simplified Cookie access'''
        return {
            key: self.raw_cookies[key].value
            for key in self.raw_cookies.keys()
        }

    @buffered_property
    def query_data(self):
        return parse_qs(
            self.environ.get('QUERY_STRING', ''),
            keep_blank_values=True
        )

    @buffered_property
    def body(self):
        size = int(self.environ.get('CONTENT_LENGTH', 0))
        if not size:
            return ''
        return self.environ['wsgi.input'].read(size)

    @buffered_property
    def request_data(self):
        if self.content_type == 'application/x-www-form-urlencoded':
            return parse_qs(self.body)
        # Support multi-part
        elif self.content_type == 'multipart/form-data':
            return cgi.parse_multipart(
                self.environ['wsgi.input'],
                self.content_params['boundary']
            )
        return {}





import logging
log = logging.getLogger(__name__)

from .request import Request

class App(object):
    '''
    Base Application class.

    Create an instance of this, passing configuration options, and use the resulting instance as your WSGI application callable.

        application = App(root_view=myview)

    You can also sub-class this to provide the root_view.
    '''
    def __init__(self, **opts):
        for key, val in opts.items():
            setattr(self, key, val)

    def __call__(self, environ, start_response):
        request = Request(self, environ)

        response = self.root_view(request)

        start_response(response.status, response.build_headers())

        return response
